import c4d
from c4d import gui
import os, glob
from os import path

# Unique id numbers for each of the GUI elements
DIRECTORY_EDIT = 10000
RECURSIVE_SEARCH = 10001
GROUP_OPTIONS = 20000
BTN_OK = 20001
BTN_CANCEL = 20002

# Dialog for importing vrscan materials.
class ImportOptions(gui.GeDialog):
    def CreateLayout(self):
        self.SetTitle('Import directory with vrscan files')
        fileData=c4d.BaseContainer()
        fileData.SetBool(c4d.FILENAME_DIRECTORY, True)
        self.AddCustomGui(DIRECTORY_EDIT, c4d.CUSTOMGUI_FILENAME, 'Directory:', c4d.BFH_SCALEFIT, 0, 0, fileData)
        self.AddCheckbox(RECURSIVE_SEARCH, c4d.BFH_SCALEFIT, 0, 0, 'Include Sub Directories')
        # Add closing buttons
        self.GroupBegin(GROUP_OPTIONS, c4d.BFH_CENTER, 2, 1)
        self.AddButton(BTN_OK, c4d.BFH_SCALE, name='OK')
        self.AddButton(BTN_CANCEL, c4d.BFH_SCALE, name='Cancel')
        self.GroupEnd()
        self.ok = False
        return True
 
    # React to user's input:
    def Command(self, id, msg):
        if id==BTN_CANCEL:
            self.Close()
        elif id==BTN_OK:
            self.ok = True
            self.optionDirectory = self.GetFilename(DIRECTORY_EDIT)
            self.optionRecursive = self.GetBool(RECURSIVE_SEARCH)
            self.Close()
        return True

def importVRScans(directory: str, recursiveSearch: bool):
    globString = path.join(directory, '*.vrscan') if not recursiveSearch else path.join(directory, '**', '*.vrscan')
    vrscanFiles = glob.glob(globString, recursive=recursiveSearch)
    if not vrscanFiles:
        raise Exception('Path does not contain any .vrscan files')
    c4d.StatusSetText('Importing VRScan files...')
    doc = c4d.documents.GetActiveDocument()
    fileCount = len(vrscanFiles)
    currentFile = 0
    # Iterate over all the vrscan files and create a material
    for vrscanFile in vrscanFiles:
        # Create a V-Ray Scanned material
        scanMaterial=c4d.BaseMaterial(1056284)
        if not scanMaterial:
            raise Exception('Failed creating a V-Ray Scanned Material')
        baseFileName=path.splitext(path.basename(vrscanFile))[0]
        scanMaterial.SetName(baseFileName)
        # Set the file name parameter
        scanMaterial.SetParameter(c4d.BRDFSCANNED_FILE, vrscanFile, c4d.DESCFLAGS_SET_NONE)
        # Insert the material and send it a message update
        doc.InsertMaterial(scanMaterial)
        scanMaterial.Message(c4d.MSG_CHANGE)
        # Update progress bar and status text to indicate usage
        currentFile = currentFile + 1
        percentDone = currentFile * 100 / fileCount
        c4d.StatusSetBar(percentDone)
        c4d.StatusSetText('Importing VRScan files... [{} of {}]'.format(currentFile, fileCount))
        # Finally make C4D update its UI
        c4d.EventAdd()
    return

# Main function
def main():
    # Open the dialog for choosing the directory for importing the vrscan files
    dialog = ImportOptions()
    dialog.Open(c4d.DLG_TYPE_MODAL, defaultw=300, defaulth=50)
    if not dialog.ok:
        return
    searchDirectory=dialog.optionDirectory
    if not searchDirectory:
        gui.MessageDialog('No directory was provided', c4d.GEMB_ICONEXCLAMATION)
        return
    if not path.exists(searchDirectory) or not path.isdir(searchDirectory):
        gui.MessageDialog('The provided directory does not exist or is not a directory', c4d.GEMB_ICONEXCLAMATION)
        return
    try:
        importVRScans(searchDirectory, dialog.optionRecursive)
    except Exception as e:
        gui.MessageDialog('Failed importing files: {}'.format(' '.join(map(str, e.args))), c4d.GEMB_ICONSTOP)
        return
    gui.MessageDialog('Finished import of directory: ' + searchDirectory)

# Execute main()
if __name__=='__main__':
    main()